# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the control panel backend DBus service."""

# pylint: disable=C0103
# DBus signals have CamelCased names

import dbus
import ubuntu_sso

from twisted.internet.defer import inlineCallbacks
from twisted.trial import unittest

from ubuntuone.controlpanel import dbus_service
from ubuntuone.controlpanel import dbus_client

SAMPLE_CREDS = {
    "token": "ABCDEF12345678",
    "access_token": "DEADCAFE2010",
}


# pylint: disable=C0322
# pylint, you have to go to decorator's school

class MockDBusSSOClient(dbus.service.Object):
    """A mock object that mimicks ussoc."""

    @dbus.service.method(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         in_signature="sssx")
    def login_or_register_to_get_credentials(self, app_name, tcurl, hlp, wid):
        """Get creds from the keyring, login/register if needed."""
        self.CredentialsFound(app_name, SAMPLE_CREDS)

    @dbus.service.signal(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         signature="sa{ss}")
    def CredentialsFound(self, app_name, credentials):
        """Credentials were finally found."""


class MockDBusSSOClientFailing(dbus.service.Object):
    """A mock object that mimicks ussoc but fails."""

    @dbus.service.method(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         in_signature="sssx")
    def login_or_register_to_get_credentials(self, app_name, tcurl, hlp, wid):
        """Fail while trying to get creds from the keyring."""
        self.CredentialsError(app_name, "error message", "error details")

    @dbus.service.signal(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         signature="sss")
    def CredentialsError(self, app_name, error_message, detailed_error):
        """Some error happened and credentials were not found."""

# pylint: enable=C0322

class MockDBusSSOClientFailingDBus(dbus.service.Object):
    """A mock that fails at the DBus layer (because it's got no methods!)."""


class DBusClientTestCase(unittest.TestCase):
    """test for the dbus client."""
    timeout = 5

    def setUp(self):
        super(DBusClientTestCase, self).setUp()
        dbus_service.init_mainloop()

    def register_mockserver(self, bus_name, object_path, object_class):
        """The mock service is registered on the DBus."""
        session_bus = dbus.SessionBus()
        name = session_bus.request_name(bus_name,
                                        dbus.bus.NAME_FLAG_DO_NOT_QUEUE)
        self.assertNotEqual(name, dbus.bus.REQUEST_NAME_REPLY_EXISTS)
        bus_name = dbus.service.BusName(bus_name, bus=dbus.SessionBus())
        mock = object_class(object_path=ubuntu_sso.DBUS_CRED_PATH,
                            bus_name=bus_name)
        self.addCleanup(mock.remove_from_connection)

    @inlineCallbacks
    def test_get_credentials_ok(self):
        """Test the success case for get_credentials."""
        self.register_mockserver(ubuntu_sso.DBUS_BUS_NAME,
                                 ubuntu_sso.DBUS_CRED_PATH, MockDBusSSOClient)
        creds = yield dbus_client.get_credentials()
        self.assertEqual(creds, SAMPLE_CREDS)

    @inlineCallbacks
    def test_get_credentials_error(self):
        """Test what happens when the creds can't be retrieved."""
        self.register_mockserver(ubuntu_sso.DBUS_BUS_NAME,
                                 ubuntu_sso.DBUS_CRED_PATH,
                                 MockDBusSSOClientFailing)

        yield self.assertFailure(dbus_client.get_credentials(),
                                 dbus_client.CredentialsError)

    @inlineCallbacks
    def test_get_credentials_dbus_error(self):
        """Test what happens when there's a DBus error."""
        self.register_mockserver(ubuntu_sso.DBUS_BUS_NAME,
                                 ubuntu_sso.DBUS_CRED_PATH,
                                 MockDBusSSOClientFailingDBus)

        yield self.assertFailure(dbus_client.get_credentials(),
                                 dbus.DBusException)
