# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the control panel backend DBus service."""

import dbus
import mocker

from twisted.internet import defer
from twisted.trial import unittest

from ubuntuone.controlpanel import dbus_service
from ubuntuone.controlpanel import (DBUS_BUS_NAME, DBUS_PREFERENCES_PATH,
                                    DBUS_PREFERENCES_IFACE)

SAMPLE_ACCOUNT_INFO = {
    "quota_used": "12345",
    "quota_total": "54321",
    "type": "paying customer",
    "name": "john carlos",
    "email": "john@carlos.com",
}

class DBusServiceMainTestCase(mocker.MockerTestCase):
    """Tests for the main function."""

    def test_dbus_service_main(self):
        """The main method starts the loop and hooks up to DBus."""
        rs_name = "ubuntuone.controlpanel.dbus_service.register_service"
        rs = self.mocker.replace(rs_name)
        rs()
        self.mocker.result(True)
        rml_name = "ubuntuone.controlpanel.dbus_service.run_mainloop"
        rml = self.mocker.replace(rml_name)
        rml()
        pb_name = "ubuntuone.controlpanel.dbus_service.publish_backend"
        pb = self.mocker.replace(pb_name)
        pb()
        self.mocker.replay()
        dbus_service.main()

    def test_dbus_service_cant_register(self):
        """The main method can't start the loop."""
        rs_name = "ubuntuone.controlpanel.dbus_service.register_service"
        rs = self.mocker.replace(rs_name)
        rs()
        self.mocker.result(False)
        self.mocker.replay()
        dbus_service.main()


class MockBackend(object):
    """A mock backend."""
    fail = False

    def account_info(self):
        """Get the user account info."""
        if self.fail:
            # XXX: use a test exception here
            return defer.fail(Exception())
        return defer.succeed(SAMPLE_ACCOUNT_INFO)


class DBusServiceTestCase(unittest.TestCase):
    """Test for the DBus service."""
    timeout = 5
    iface_name = ""

    def setUp(self):
        """Initialize each test run."""
        super(DBusServiceTestCase, self).setUp()
        dbus_service.init_mainloop()

    def test_register_service(self):
        """The DBus service is successfully registered."""
        ret = dbus_service.register_service()
        self.assertTrue(ret)

    def test_cant_register_twice(self):
        """The DBus service can't register if it already did."""
        ret = dbus_service.register_service()
        self.assertTrue(ret)
        ret = dbus_service.register_service()
        self.assertFalse(ret)
    #pylint: disable=W0612
    test_cant_register_twice.skip = "Must run 2nd check in another process."

    def test_dbus_busname_created(self):
        """The DBus BusName is created."""
        busname = dbus_service.get_busname()
        self.assertEqual(busname.get_name(), DBUS_BUS_NAME)

    def test_account_info_returned(self):
        """The account info is successfully returned."""
        be = dbus_service.publish_backend(MockBackend())
        self.addCleanup(be.remove_from_connection)

        bus = dbus.SessionBus()
        obj = bus.get_object(bus_name=DBUS_BUS_NAME,
                             object_path=DBUS_PREFERENCES_PATH,
                             follow_name_owner_changes=True)
        self.iface_name = DBUS_PREFERENCES_IFACE
        backend = dbus.Interface(object=obj,
                                 dbus_interface=DBUS_PREFERENCES_IFACE)
        d = defer.Deferred()

        def got_signal(account_info):
            """The correct signal was fired."""
            self.assertIn("quota_used", account_info)
            self.assertIn("quota_total", account_info)
            self.assertIn("type", account_info)
            self.assertIn("name", account_info)
            self.assertIn("email", account_info)
            d.callback("success")

        def got_error(*a):
            """Some error happened in the DBus call."""
            d.errback(*a)

        def ignore(*a):
            """Do nothing with the returned value."""

        backend.connect_to_signal("AccountInfoReady", got_signal)

        backend.account_info(reply_handler=ignore, error_handler=got_error)
        return d

    def test_devices_info_returned(self):
        """The devices info is successfully returned."""
        be = dbus_service.publish_backend()
        self.addCleanup(be.remove_from_connection)

        bus = dbus.SessionBus()
        obj = bus.get_object(bus_name=DBUS_BUS_NAME,
                             object_path=DBUS_PREFERENCES_PATH,
                             follow_name_owner_changes=True)
        self.iface_name = DBUS_PREFERENCES_IFACE
        backend = dbus.Interface(object=obj,
                                 dbus_interface=DBUS_PREFERENCES_IFACE)
        d = defer.Deferred()

        def got_signal(devices_list):
            """The correct signal was fired."""
            for device_info in devices_list:
                self.assertIn("device_token", device_info)
                self.assertIn("name", device_info)
                self.assertIn("date_added", device_info)
                self.assertIn("type", device_info)
                self.assertIn("configurable", device_info)
                self.assertIn("limit_bandwidth", device_info)
                self.assertIn("max_upload_speed", device_info)
                self.assertIn("max_download_speed", device_info)
                self.assertIn("available_services", device_info)
                self.assertIn("enabled_services", device_info)
            d.callback("success")

        def got_error(*a):
            """Some error happened in the DBus call."""
            d.errback(*a)

        def ignore(*a):
            """Do nothing with the returned value."""

        backend.connect_to_signal("DevicesInfoReady", got_signal)

        backend.devices_info(reply_handler=ignore, error_handler=got_error)
        return d

    def test_change_device_setting(self):
        """The devices settings are successfully changed."""
        be = dbus_service.publish_backend()
        self.addCleanup(be.remove_from_connection)
        sample_token = "token-1"

        bus = dbus.SessionBus()
        obj = bus.get_object(bus_name=DBUS_BUS_NAME,
                             object_path=DBUS_PREFERENCES_PATH,
                             follow_name_owner_changes=True)
        self.iface_name = DBUS_PREFERENCES_IFACE
        backend = dbus.Interface(object=obj,
                                 dbus_interface=DBUS_PREFERENCES_IFACE)
        d = defer.Deferred()

        def got_signal(token):
            """The correct token was received."""
            self.assertEqual(token, sample_token)
            d.callback("success")

        def got_error(*a):
            """Some error happened in the DBus call."""
            d.errback(*a)

        def ignore(*a):
            """Do nothing with the returned value."""

        settings = {
            "enabled_services": "files, contacts",
        }
        backend.connect_to_signal("DeviceSettingsChanged", got_signal)
        backend.change_device_settings(sample_token, settings,
                                       reply_handler=ignore,
                                       error_handler=got_error)
        return d

    def test_remove_device(self):
        """The device is removed."""
        be = dbus_service.publish_backend()
        self.addCleanup(be.remove_from_connection)
        sample_token = "token-1"

        bus = dbus.SessionBus()
        obj = bus.get_object(bus_name=DBUS_BUS_NAME,
                             object_path=DBUS_PREFERENCES_PATH,
                             follow_name_owner_changes=True)
        self.iface_name = DBUS_PREFERENCES_IFACE
        backend = dbus.Interface(object=obj,
                                 dbus_interface=DBUS_PREFERENCES_IFACE)
        d = defer.Deferred()

        def got_signal(token):
            """The correct token was received."""
            self.assertEqual(token, sample_token)
            d.callback("success")

        def got_error(*a):
            """Some error happened in the DBus call."""
            d.errback(*a)

        def ignore(*a):
            """Do nothing with the returned value."""

        backend.connect_to_signal("DeviceRemoved", got_signal)
        backend.remove_device(sample_token,
                              reply_handler=ignore,
                              error_handler=got_error)
        return d

    def test_file_sync_status(self):
        """The file sync status is reported."""
        be = dbus_service.publish_backend()
        self.addCleanup(be.remove_from_connection)

        bus = dbus.SessionBus()
        obj = bus.get_object(bus_name=DBUS_BUS_NAME,
                             object_path=DBUS_PREFERENCES_PATH,
                             follow_name_owner_changes=True)
        self.iface_name = DBUS_PREFERENCES_IFACE
        backend = dbus.Interface(object=obj,
                                 dbus_interface=DBUS_PREFERENCES_IFACE)
        d = defer.Deferred()

        def got_signal(enabled, status):
            """The correct status was received."""
            self.assertEqual(enabled, True)
            self.assertEqual(status, "Synchronizing")
            d.callback("success")

        def got_error(*a):
            """Some error happened in the DBus call."""
            d.errback(*a)

        def ignore(*a):
            """Do nothing with the returned value."""

        backend.connect_to_signal("FileSyncStatusReady", got_signal)
        backend.file_sync_status(reply_handler=ignore,
                                 error_handler=got_error)
        return d

    def test_query_bookmarks_extension(self):
        """The bookmarks extension is queried."""
        be = dbus_service.publish_backend()
        self.addCleanup(be.remove_from_connection)

        bus = dbus.SessionBus()
        obj = bus.get_object(bus_name=DBUS_BUS_NAME,
                             object_path=DBUS_PREFERENCES_PATH,
                             follow_name_owner_changes=True)
        self.iface_name = DBUS_PREFERENCES_IFACE
        backend = dbus.Interface(object=obj,
                                 dbus_interface=DBUS_PREFERENCES_IFACE)
        d = defer.Deferred()

        def got_signal(enabled):
            """The correct status was received."""
            self.assertEqual(enabled, False)
            d.callback("success")

        def got_error(*a):
            """Some error happened in the DBus call."""
            d.errback(*a)

        def ignore(*a):
            """Do nothing with the returned value."""

        backend.connect_to_signal("QueryBookmarksResult", got_signal)
        backend.query_bookmark_extension(reply_handler=ignore,
                                         error_handler=got_error)
        return d

    def test_install_bookmarks_extension(self):
        """The bookmarks extension is installed."""
        be = dbus_service.publish_backend()
        self.addCleanup(be.remove_from_connection)

        bus = dbus.SessionBus()
        obj = bus.get_object(bus_name=DBUS_BUS_NAME,
                             object_path=DBUS_PREFERENCES_PATH,
                             follow_name_owner_changes=True)
        self.iface_name = DBUS_PREFERENCES_IFACE
        backend = dbus.Interface(object=obj,
                                 dbus_interface=DBUS_PREFERENCES_IFACE)
        d = defer.Deferred()

        def got_signal():
            """The extension was installed."""
            d.callback("success")

        def got_error(*a):
            """Some error happened in the DBus call."""
            d.errback(*a)

        def ignore(*a):
            """Do nothing with the returned value."""

        backend.connect_to_signal("InstallBookmarksSuccess", got_signal)
        backend.install_bookmarks_extension(reply_handler=ignore,
                                            error_handler=got_error)
        return d
