# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Export the control backend thru DBus."""

import dbus.service
import gobject

from dbus.mainloop.glib import DBusGMainLoop
from dbus.service import method, signal

from ubuntuone.controlpanel import (DBUS_BUS_NAME, DBUS_PREFERENCES_PATH,
                                    DBUS_PREFERENCES_IFACE)
from ubuntuone.controlpanel.backend import ControlBackend

# XXX: should get this info from the web service
SAMPLE_DEVICES_INFO = [
    {
        "device_token": "token-1",
        "name": "Ubuntu One @ darkstar",
        "date_added": "2008-12-06T18:15:38.0",
        "type": "computer",
        "configurable": "1",
        "limit_bandwidth": "1",
        "max_upload_speed": "12345",
        "max_download_speed": "54321",
        "available_services": "files, contacts, music, bookmarks",
        "enabled_services": "files, music",
    },
    {
        "device_token": "token-2",
        "name": "Ubuntu One @ brightmoon",
        "date_added": "2010-09-22T20:45:38.0",
        "type": "computer",
        "configurable": "0",
        "limit_bandwidth": "0",
        "max_upload_speed": "100000",
        "max_download_speed": "100000",
        "available_services": "files, contacts, bookmarks",
        "enabled_services": "files, bookmarks",
    },
]


class ControlPanelBackend(dbus.service.Object):
    """Export the Control Panel backend thru DBus."""

    def __init__(self, backend, *args, **kwargs):
        """Create this instance of the backend."""
        super(ControlPanelBackend, self).__init__(*args, **kwargs)
        self.backend = backend

    # pylint: disable=C0103

    @method(dbus_interface=DBUS_PREFERENCES_IFACE, in_signature="")
    def account_info(self):
        """Find out the account info for the current logged in user."""
        # pylint: disable=E1101
        d = self.backend.account_info()
        d.addCallback(self.AccountInfoReady)
        d.addErrback(self.AccountInfoError)

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="a{ss}")
    def AccountInfoReady(self, info):
        """The info for the current user is available right now."""

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="a{ss}")
    def AccountInfoError(self, error):
        """The info for the current user is currently unavailable."""

    #---

    @method(dbus_interface=DBUS_PREFERENCES_IFACE, in_signature="")
    def devices_info(self):
        """Find out the devices info for the logged in user."""
        self.DevicesInfoReady(SAMPLE_DEVICES_INFO)

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="aa{ss}")
    def DevicesInfoReady(self, info):
        """The info for the devices is available right now."""

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="a{ss}")
    def DevicesInfoError(self, error):
        """The info for the devices is currently unavailable."""

    #---

    @method(dbus_interface=DBUS_PREFERENCES_IFACE, in_signature="sa{ss}")
    def change_device_settings(self, token, settings):
        """Configure a given device."""
        self.DeviceSettingsChanged(token)

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="s")
    def DeviceSettingsChanged(self, token):
        """The settings for the device were changed."""

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="a{ss}")
    def DeviceSettingsChangeError(self, error):
        """Problem changing settings for the device."""

    #---

    @method(dbus_interface=DBUS_PREFERENCES_IFACE, in_signature="s")
    def remove_device(self, token):
        """Remove a given device."""
        self.DeviceRemoved(token)

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="s")
    def DeviceRemoved(self, token):
        """The removal for the device was completed."""

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="a{ss}")
    def DeviceRemovalError(self, error):
        """Problem removing the device."""

    #---

    @method(dbus_interface=DBUS_PREFERENCES_IFACE, in_signature="")
    def file_sync_status(self):
        """Get the status of the file sync service."""
        self.FileSyncStatusReady(True, "Synchronizing")

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="bs")
    def FileSyncStatusReady(self, enabled, status):
        """The new file sync status is available."""

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="a{ss}")
    def FileSyncStatusError(self, error):
        """Problem getting the file sync status."""

    #---

    @method(dbus_interface=DBUS_PREFERENCES_IFACE, in_signature="")
    def query_bookmark_extension(self):
        """Check if the extension to sync bookmarks is installed."""
        self.QueryBookmarksResult(False)

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="b")
    def QueryBookmarksResult(self, enabled):
        """The bookmark extension is or is not installed."""

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="a{ss}")
    def QueryBookmarksError(self, error):
        """Problem getting the status of the extension."""

    #---

    @method(dbus_interface=DBUS_PREFERENCES_IFACE, in_signature="")
    def install_bookmarks_extension(self):
        """Install the extension to sync bookmarks."""
        self.InstallBookmarksSuccess()

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="")
    def InstallBookmarksSuccess(self):
        """The extension to sync bookmarks has been installed."""

    @signal(dbus_interface=DBUS_PREFERENCES_IFACE, signature="a{ss}")
    def InstallBookmarksError(self, error):
        """Problem installing the extension to sync bookmarks."""


def init_mainloop():
    """Start the DBus mainloop."""
    DBusGMainLoop(set_as_default=True)


def run_mainloop():
    """Run the gobject main loop."""
    loop = gobject.MainLoop()
    loop.run()


def register_service():
    """Try to register DBus service for making sure we run only one instance.

    Return True if succesfully registered, False if already running.
    """
    session_bus = dbus.SessionBus()
    name = session_bus.request_name(DBUS_BUS_NAME,
                                    dbus.bus.NAME_FLAG_DO_NOT_QUEUE)
    return name != dbus.bus.REQUEST_NAME_REPLY_EXISTS


def get_busname():
    """Build the DBus BusName."""
    return dbus.service.BusName(DBUS_BUS_NAME, bus=dbus.SessionBus())


def publish_backend(backend=ControlBackend()):
    """Publish the backend on the DBus."""
    return ControlPanelBackend(backend=backend,
                               object_path=DBUS_PREFERENCES_PATH,
                               bus_name=get_busname())


def main():
    """Hook the DBus listeners and start the main loop."""
    init_mainloop()
    if register_service():
        publish_backend()
        run_mainloop()
    else:
        print "Control panel backend already running."
