# -*- coding: utf-8 -*-

# Authors: Natalia B Bidart <natalia.bidart@canonical.com>
#
# Copyright 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The user interface for the control panel for Ubuntu One."""

from __future__ import division

import gettext

from functools import wraps

import dbus
import gtk
import gobject
import ubuntu_sso

from ubuntu_sso import networkstate
from ubuntu_sso.credentials import (TC_URL_KEY, HELP_TEXT_KEY, WINDOW_ID_KEY,
    PING_URL_KEY)
# No name 'clientdefs' in module 'ubuntuone'
# pylint: disable=E0611,F0401
from gi.repository import GLib
from ubuntuone.clientdefs import (APP_NAME as U1_APP_NAME, TC_URL as U1_TC_URL,
    PING_URL as U1_PING_URL, DESCRIPTION as U1_DESCRIPTION)
# pylint: enable=E0611,F0401

# XXX: use ubiquity package when ready
from ubuntuone.controlpanel.widgets import GreyableBin

from ubuntuone.controlpanel import (DBUS_BUS_NAME, DBUS_PREFERENCES_PATH,
    DBUS_PREFERENCES_IFACE)
from ubuntuone.controlpanel.logger import setup_logging
from ubuntuone.controlpanel.utils import get_data_file, log_call


logger = setup_logging('gui')
_ = gettext.gettext


# XXX: To be replaced by values from the theme or Ubuntu One' specific
ORANGE = '#c95724'
LOADING = _('Loading...')
OVERVIEW_MSGS = [
    _('<b>Ubuntu One</b> Music Store. '
      'A cloud enabled mobile shopping experience.\n'
      '<small>«this is custom text #1»</small>'),
    _('<i>Mobile</i> contacts sync. '
      '<span foreground="red">Your portable address book.</span>\n'
      '<small>«this is custom text #3»</small>'),
    _('...'),
    _('Sync data between computers.\n'
      '<small>«this is custom text #<i>N</i>»</small>')]
WARNING_MARKUP = '<span foreground="%s"><b>%%s</b></span>' % ORANGE


def filter_by_app_name(f):
    """Excecute 'f' filtering by app_name."""

    @wraps(f)
    def filter_by_app_name_inner(instance, app_name, *args, **kwargs):
        """Execute 'f' only if 'app_name' matches 'U1_APP_NAME'."""
        if app_name == U1_APP_NAME:
            return f(instance, app_name, *args, **kwargs)
        else:
            logger.info('%s: ignoring call since received app_name '\
                        '"%s" (expected "%s")',
                        f.__name__, app_name, U1_APP_NAME)
    return filter_by_app_name_inner


class Loading(gtk.HBox):
    """A spinner and a label."""

    def __init__(self, *args, **kwargs):
        super(Loading, self).__init__(*args, **kwargs)
        self.label = gtk.Label(LOADING)
        self.spinner = gtk.Spinner()
        self.spinner.start()

        self.pack_start(self.spinner, expand=False)
        self.pack_start(self.label, expand=False)
        self.set_spacing(5)

        self.show_all()


class LabelLoading(gtk.HBox):
    """A spinner and a label."""

    def __init__(self, *args, **kwargs):
        super(LabelLoading, self).__init__(*args, **kwargs)
        self.loading = Loading()
        self.label = gtk.Label()
        self.pack_start(self.label, expand=False)
        self.pack_start(self.loading, expand=False)

        self.show()
        self.start()

    @property
    def active(self):
        """Whether the Loading widget is visible or not."""
        return not self.label.get_visible() and self.loading.get_visible()

    def start(self):
        """Show the Loading instead of the Label widget."""
        self.label.hide()
        self.loading.show()

    def stop(self):
        """Show the label instead of the Loading widget."""
        self.label.show()
        self.loading.hide()

    def set_text(self, text):
        """Set 'text' to be the label's text."""
        self.label.set_text(text)

    def set_markup(self, text):
        """Set 'text' to be the label's markup."""
        self.label.set_markup(text)

    def get_text(self):
        """Get the label's text."""
        return self.label.get_text()

    def get_label(self):
        """Get the label's markup."""
        return self.label.get_label()


class ControlPanelMixin(object):
    """The main interface for the Ubuntu One control panel."""

    VALUE_ERROR = _('Value could not be retrieved.')

    def __init__(self, filename=None):
        bus = dbus.SessionBus()
        try:
            obj = bus.get_object(DBUS_BUS_NAME,
                                 DBUS_PREFERENCES_PATH,
                                 follow_name_owner_changes=True)
            iface = DBUS_PREFERENCES_IFACE
            self.backend = dbus.Interface(obj, dbus_interface=iface)
        except dbus.exceptions.DBusException:
            logger.exception('Can not connect to DBus at %r',
                             (DBUS_BUS_NAME, DBUS_PREFERENCES_PATH))
            raise

        self.builder = gtk.Builder()
        if filename is not None:
            self.builder.add_from_file(get_data_file(filename))
            self.builder.connect_signals(self)

            # untested directly
            for obj in self.builder.get_objects():
                name = getattr(obj, 'name', None)
                if name is None and isinstance(obj, gtk.Buildable):
                    # work around bug lp:507739
                    name = gtk.Buildable.get_name(obj)
                if name is None:
                    logger.warning("%s has no name (??)", obj)
                else:
                    setattr(self, name, obj)

        logger.debug('%s: started.', self.__class__.__name__)

    def debug(self, *args, **kwargs):
        """Do some debug."""
        print '\n', args, kwargs

    def humanize(self, int_bytes):
        """Return a human readble string of 'int_bytes'."""
        return GLib.format_size_for_display(int_bytes)

    def _set_warning(self, message, label):
        """Set 'message' as warning in 'label'."""
        label.set_markup(WARNING_MARKUP % message)
        label.show()

    def on_size_allocate(self, label, allocation):
        """The label can re rezised, embrase it!."""
        label.set_size_request(allocation.width, -1)

    #def __getattribute__(self, attr):
    #    result = None
    #    try:
    #        itself = super(ControlPanelMixin, self).__getattribute__('itself')
    #        result = getattr(itself, attr)
    #    except AttributeError:
    #        pass
    #    if result is None:
    #        print '!!!!!!!!!! No attr %s in itself, trying self.' % attr
    #        result = super(ControlPanelMixin, self).__getattribute__(attr)
    #    print '********** Final result for %r is %s.' % (attr, result)
    #    return result


class ControlPanelWindow(gtk.Window):
    """The main window for the Ubuntu One control panel."""

    TITLE = _('My %(app_name)s Account')

    def __init__(self):
        super(ControlPanelWindow, self).__init__()

        self.set_title(self.TITLE % {'app_name': U1_APP_NAME})
        self.set_position(gtk.WIN_POS_CENTER_ALWAYS)
        self.set_icon_name('ubuntuone')
        self.set_size_request(815, 576)
        self.connect('delete-event', lambda w, e: gtk.main_quit())
        self.show()

        self.control_panel = ControlPanel(window_id=self.window.xid)
        self.add(self.control_panel)

        logger.debug('%s: started (window size %r).',
                     self.__class__.__name__, self.get_size_request())

    def main(self):
        """Run the main loop of the widget toolkit."""
        logger.debug('Starting GTK main loop.')
        gtk.main()


class ControlPanel(gtk.VBox, ControlPanelMixin):
    """The control panel per se, can be added into any other widget."""

    def __init__(self, window_id=0):
        gtk.VBox.__init__(self)
        ControlPanelMixin.__init__(self, filename='controlpanel.ui')
        self._window_id = window_id

        self.overview = OverviewPanel(window_id=window_id)
        self.overview.connect('credentials-found',
                              self.on_show_management_panel)
        self.add(self.overview)
        self.show()

    def on_show_management_panel(self, *args, **kwargs):
        """Show the netbook (main panel)."""
        self.add(ManagementPanel())


class UbuntuOneBin(GreyableBin, ControlPanelMixin):
    """A Ubuntu One bin."""

    def __init__(self):
        GreyableBin.__init__(self)
        ControlPanelMixin.__init__(self, filename='page.ui')
        self.add(self.itself)
        self.show()


class OverviewPanel(GreyableBin, ControlPanelMixin):
    """The overview panel. Introduces Ubuntu One to the not logged user."""

    CREDENTIALS_ERROR = _('There was a problem while retrieving the '
        'credentials.')
    AUTHORIZATION_DENIED = _('The authentication was cancelled. Please either '
        'click join or connect to continue.')
    NETWORK_OFFLINE = _('An internet connection is required to join or sign '
        'in to %(app_name)s.')
    NETWORK_UNKNOWN = _('The internet connection state couldn\'t be '
        'discovered. Maybe NetworkManager is not running?')

    BULLET = '<span foreground="%s">✔</span>' % ORANGE

    def __init__(self, messages=None, window_id=0):
        gtk.link_button_set_uri_hook(lambda *a: None)
        GreyableBin.__init__(self)
        ControlPanelMixin.__init__(self, filename='overview.ui')
        self.add(self.itself)
        self.warning_label.hide()
        self.warning_label.set_property('xalign', 0.5)
        self.warning_label.connect('size-allocate', self.on_size_allocate)

        self._messages = messages
        self._window_id = window_id
        self._build_messages()
        self.show()

        bus = dbus.SessionBus()
        try:
            obj = bus.get_object(ubuntu_sso.DBUS_BUS_NAME,
                                 ubuntu_sso.DBUS_CREDENTIALS_PATH,
                                 follow_name_owner_changes=True)
            iface = ubuntu_sso.DBUS_CREDENTIALS_IFACE
            self.sso_backend = dbus.Interface(obj, dbus_interface=iface)
        except dbus.exceptions.DBusException:
            logger.exception('Can not connect to DBus at %r',
                             (ubuntu_sso.DBUS_BUS_NAME,
                              ubuntu_sso.DBUS_CREDENTIALS_PATH))
            raise

        # TODO: disconnect signals
        self.sso_backend.connect_to_signal('CredentialsFound',
                                           self.on_credentials_found)
        self.sso_backend.connect_to_signal('CredentialsNotFound',
                                           self.on_credentials_not_found)
        self.sso_backend.connect_to_signal('CredentialsError',
                                           self.on_credentials_error)
        self.sso_backend.connect_to_signal('AuthorizationDenied',
                                           self.on_authorization_denied)
        kw = dict(result_cb=self.on_network_state_changed)
        self.network_manager_state = networkstate.NetworkManagerState(**kw)
        self.network_manager_state.find_online_state()

    def _build_messages(self):
        """List messages in a itemized list."""
        if self._messages is None:
            self._messages = OVERVIEW_MSGS

        for msg in self._messages:
            label = gtk.Label()
            label.set_markup(self.BULLET + ' ' + msg)
            label.set_property('xalign', 0)
            label.set_property('wrap', True)
            label.connect('size-allocate', self.on_size_allocate)

            self.messages.pack_start(label)
        self.messages.show_all()

    def _set_warning(self, message, label=None):
        """Set 'message' as global warning."""
        ControlPanelMixin._set_warning(self, message,
                                       label=self.warning_label)

    def set_property(self, prop_name, new_value):
        """Override 'set_property' to disable buttons if prop is 'greyed'."""
        if prop_name == 'greyed':
            self.set_sensitive(not new_value)
        GreyableBin.set_property(self, prop_name, new_value)

    def set_sensitive(self, value):
        """Set the sensitiveness as per 'value'."""
        self.join_now_button.set_sensitive(value)
        self.connect_button.set_sensitive(value)

    def get_sensitive(self):
        """Return the sensitiveness."""
        result = self.join_now_button.get_sensitive() and \
                 self.connect_button.get_sensitive()
        return result

    def on_join_now_button_clicked(self, *a, **kw):
        """User wants to join now."""
        settings = {TC_URL_KEY: U1_TC_URL, HELP_TEXT_KEY: U1_DESCRIPTION,
                    WINDOW_ID_KEY: str(self._window_id),
                    PING_URL_KEY: U1_PING_URL}
        self.sso_backend.register(U1_APP_NAME, settings)
        self.set_property('greyed', True)
        self.warning_label.hide()

    def on_connect_button_clicked(self, *a, **kw):
        """User wants to connect now."""
        settings = {TC_URL_KEY: U1_TC_URL, HELP_TEXT_KEY: U1_DESCRIPTION,
                    WINDOW_ID_KEY: str(self._window_id),
                    PING_URL_KEY: U1_PING_URL}
        self.sso_backend.login(U1_APP_NAME, settings)
        self.set_property('greyed', True)
        self.warning_label.hide()

    @filter_by_app_name
    @log_call(logger.info)
    def on_credentials_found(self, app_name, credentials):
        """SSO backend notifies of credentials found."""
        self.emit('credentials-found', app_name, credentials)
        self.hide()

    @filter_by_app_name
    @log_call(logger.info)
    def on_credentials_not_found(self, app_name):
        """SSO backend notifies of credentials not found."""
        self.set_property('greyed', False)

    @filter_by_app_name
    @log_call(logger.error)
    def on_credentials_error(self, app_name, error_dict):
        """SSO backend notifies of an error when fetching credentials."""
        self.set_property('greyed', False)
        self._set_warning(self.CREDENTIALS_ERROR)

    @filter_by_app_name
    @log_call(logger.info)
    def on_authorization_denied(self, app_name):
        """SSO backend notifies that user refused auth for 'app_name'."""
        self.set_property('greyed', False)
        self._set_warning(self.AUTHORIZATION_DENIED)

    @log_call(logger.info)
    def on_network_state_changed(self, state):
        """Network state is reported."""
        msg = None
        if state is networkstate.OFFLINE:
            msg = self.NETWORK_OFFLINE % {'app_name': U1_APP_NAME}
        elif state is networkstate.UNKNOWN:
            msg = self.NETWORK_UNKNOWN

        if msg is not None:
            self.set_sensitive(False)
            self._set_warning(msg)
        else:
            self.set_sensitive(True)
            self.warning_label.hide()
            self.sso_backend.find_credentials(U1_APP_NAME, {})


class AccountPanel(gtk.VBox, ControlPanelMixin):
    """The account panel. The user can manage the subscription."""

    def __init__(self):
        gtk.VBox.__init__(self)
        ControlPanelMixin.__init__(self, filename='account.ui')
        self.pack_start(self.itself)
        self.show()

        self.backend.connect_to_signal('AccountInfoReady',
                                       self.on_account_info_ready)
        self.backend.connect_to_signal('AccountInfoError',
                                       self.on_account_info_error)

        self.name_label = LabelLoading()
        self.name_box.pack_start(self.name_label)

        self.type_label = LabelLoading()
        self.type_box.pack_start(self.type_label)

        self.email_label = LabelLoading()
        self.email_box.pack_start(self.email_label)

    @log_call(logger.info)
    def on_account_info_ready(self, info):
        """Backend notifies of account info."""
        for i in ('name', 'type', 'email'):
            label = getattr(self, '%s_label' % i)
            label.set_markup('<b>%s</b>' % info[i])
            label.stop()

    @log_call(logger.error)
    def on_account_info_error(self, error_dict=None):
        """Backend notifies of an error when fetching account info."""
        for i in ('name', 'type', 'email'):
            label = getattr(self, '%s_label' % i)
            label.set_markup(WARNING_MARKUP % self.VALUE_ERROR)
            label.stop()


class ContactsPanel(gtk.VBox, ControlPanelMixin):
    """The contacts panel."""

    def __init__(self):
        gtk.VBox.__init__(self)
        ControlPanelMixin.__init__(self, filename='contacts.ui')
        self.pack_start(self.itself)
        self.show()


class DevicesPanel(gtk.VBox, ControlPanelMixin):
    """The devices panel."""

    def __init__(self):
        gtk.VBox.__init__(self)
        ControlPanelMixin.__init__(self, filename='devices.ui')
        self.pack_start(self.itself)
        self.show()


class ApplicationsPanel(gtk.VBox, ControlPanelMixin):
    """The applications panel."""

    def __init__(self):
        gtk.VBox.__init__(self)
        ControlPanelMixin.__init__(self, filename='applications.ui')
        self.pack_start(self.itself)
        self.show()


class ManagementPanel(gtk.VBox, ControlPanelMixin):
    """The management panel.

    The user can manage account, contacts, devices and applications.

    """

    QUOTA_LABEL = _('%(used)s used of %(total)s (%(percentage).1f%%)')

    def __init__(self):
        gtk.VBox.__init__(self)
        ControlPanelMixin.__init__(self, filename='management.ui')
        self.pack_start(self.itself)
        self.show()

        self.backend.connect_to_signal('AccountInfoReady',
                                       self.on_account_info_ready)
        self.backend.connect_to_signal('AccountInfoError',
                                       self.on_account_info_error)

        self.quota_label = LabelLoading()
        self.status_label = LabelLoading()
        self.status_box.pack_start(self.quota_label, expand=False)
        self.status_box.pack_end(self.status_label, expand=False)

        self.account = AccountPanel()
        self.contacts = ContactsPanel()
        self.devices = DevicesPanel()
        self.applications = ApplicationsPanel()

        cb = lambda button, page_num: self.notebook.set_current_page(page_num)
        self.tabs = (u'account', u'contacts', u'devices', u'applications')
        for page_num, tab in enumerate(self.tabs):
            setattr(self, ('%s_page' % tab).upper(), page_num)
            getattr(self, '%s_button' % tab).connect('clicked', cb, page_num)
            self.notebook.insert_page(getattr(self, tab), position=page_num)

        self.backend.account_info()

    def _update_quota(self, msg):
        """Update the quota info."""
        self.quota_label.set_markup(msg)
        self.quota_label.stop()

    @log_call(logger.info)
    def on_account_info_ready(self, info):
        """Backend notifies of account info."""
        used = int(info['quota_used'])
        total = int(info['quota_total'])
        data = {'used': self.humanize(used), 'total': self.humanize(total),
                'percentage': (used / total) * 100}
        self._update_quota(self.QUOTA_LABEL % data)

    @log_call(logger.error)
    def on_account_info_error(self, error_dict=None):
        """Backend notifies of an error when fetching account info."""
        self._update_quota(WARNING_MARKUP % self.VALUE_ERROR)


gobject.signal_new('credentials-found', OverviewPanel,
                    gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
                   (gobject.TYPE_STRING, gobject.TYPE_PYOBJECT))
