# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the control panel backend."""

import simplejson

from twisted.internet import defer
from twisted.internet.defer import inlineCallbacks
from twisted.trial.unittest import TestCase

from ubuntuone.controlpanel import backend
from ubuntuone.controlpanel.webclient import WebClientError

SAMPLE_ACCOUNT_JSON = """
{
    "username": "andrewpz",
    "openid": "https://login.launchpad.net/+id/abcdefg",
    "first_name": "Andrew P.",
    "last_name": "Zoilo",
    "couchdb": {
        "host": "https://couchdb.one.ubuntu.com",
        "root": "https://couchdb.one.ubuntu.com/u/abc/def/12345",
        "dbpath": "u/abc/def/12345"
    },
    "couchdb_root": "https://couchdb.one.ubuntu.com/u/abc/def/12345",
    "email": "andrewpz@protocultura.net",
    "nickname": "Andrew P. Zoilo",
    "id": 12345,
    "subscription": {
        "upgrade_available": false,
        "description": "Paid Plan, 50 GB of storage",
        "trial": false,
        "started": "2010-03-24T18:38:38Z",
        "is_paid": true,
        "expires": null,
        "qty": 1,
        "price": 0.0,
        "currency": null,
        "id": 654321,
        "name": "50 GB"
    }
}
"""

SAMPLE_QUOTA_JSON = """
{
    "total": 53687091200,
    "used": 2350345156
}
"""

EXPECTED_ACCOUNT_INFO = {
    "quota_used": "2350345156",
    "quota_total": "53687091200",
    "type": "Paid Plan, 50 GB of storage",
    "name": "Andrew P. Zoilo",
    "email": "andrewpz@protocultura.net",
}


class MockWebClient(object):
    """A mock webclient."""
    failure = False
    results = {}

    def __init__(self, get_credentials):
        """Initialize this mock instance."""
        self.get_credentials = get_credentials

    def call_api(self, method):
        """Get a given url from the webservice."""
        if self.failure:
            return defer.fail(WebClientError(self.failure))
        else:
            return defer.succeed(self.results[method])


class BackendBasicTestCase(TestCase):
    """Simple tests for the backend."""
    timeout = 3

    def setUp(self):
        self.patch(backend, "WebClient", MockWebClient)

    def test_backend_creation(self):
        """The backend instance is successfully created."""
        be = backend.ControlBackend()
        self.assertEqual(be.wc.__class__, MockWebClient)

    @inlineCallbacks
    def test_account_info(self):
        """The account_info method exercises its callback."""
        be = backend.ControlBackend()
        # pylint: disable=E1101
        be.wc.results["account/"] = simplejson.loads(SAMPLE_ACCOUNT_JSON)
        be.wc.results["quota/"] = simplejson.loads(SAMPLE_QUOTA_JSON)
        result = yield be.account_info()
        self.assertEqual(result, EXPECTED_ACCOUNT_INFO)

    @inlineCallbacks
    def test_account_info_fails(self):
        """The account_info method exercises its errback."""
        be = backend.ControlBackend()
        # pylint: disable=E1101
        be.wc.failure = 404
        yield self.assertFailure(be.account_info(), WebClientError)
