# -*- coding: utf-8 -*-

# Authors: Natalia B Bidart <natalia.bidart@canonical.com>
#
# Copyright 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The test suite for the control panel user interface."""

from __future__ import division

import logging

from collections import defaultdict

from twisted.trial.unittest import TestCase

from ubuntuone.controlpanel import gui
from ubuntuone.controlpanel.tests import TOKEN, testcase
# XXX: to be replaced when MementoHandler is there
#from ubuntuone.devtools import testcase

# Attribute 'yyy' defined outside __init__
# pylint: disable=W0201

# Access to a protected member 'yyy' of a client class
# pylint: disable=W0212


FAKE_ACCOUNT_INFO = {'type': 'Payed', 'name': 'Test me',
    'email': 'test.com', 'quota_total': '12345', 'quota_used': '9999'}


class FakedBuilder(gui.gtk.Builder):
    """A faked builder."""

    def add_from_file(self, filename, *a, **kw):
        """Override Builder to store 'filename'."""
        self.filename = filename
        super(FakedBuilder, self).add_from_file(filename, *a, **kw)

    def connect_signals(self, instance, *a, **kw):
        """Override Builder to store 'instance'."""
        self.signals = instance
        super(FakedBuilder, self).connect_signals(instance, *a, **kw)


class FakedObject(object):
    """Fake an object, record every call."""

    exposed_methods = []

    def __init__(self, *args, **kwargs):
        self._args = args
        self._kwargs = kwargs
        self._called = {}
        for i in self.exposed_methods:
            setattr(self, i, self._record_call(i))

    def _record_call(self, func_name):
        """Store values when calling 'func_name'."""

        def inner(*args, **kwargs):
            """Fake 'func_name'."""
            self._called[func_name] = (args, kwargs)

        return inner


class FakedNMState(FakedObject):
    """Fake a NetworkManagerState."""

    exposed_methods = ['find_online_state']


class FakedDBusBackend(FakedObject):
    """Fake a DBus Backend."""

    bus_name = None
    object_path = None
    iface = None

    def __init__(self, obj, dbus_interface, *args, **kwargs):
        if dbus_interface != self.iface:
            raise TypeError()
        self._signals = defaultdict(list)
        super(FakedDBusBackend, self).__init__(*args, **kwargs)

    def connect_to_signal(self, signal, handler):
        """Bind 'handler' to be callback'd when 'signal' is fired."""
        self._signals[signal].append(handler)


class FakedSSOBackend(FakedDBusBackend):
    """Fake a SSO Backend, act as a dbus.Interface."""

    bus_name = gui.ubuntu_sso.DBUS_BUS_NAME
    object_path = gui.ubuntu_sso.DBUS_CREDENTIALS_PATH
    iface = gui.ubuntu_sso.DBUS_CREDENTIALS_IFACE
    exposed_methods = ['find_credentials', 'clear_credentials',
                       'login', 'register']


class FakedControlPanelBackend(FakedDBusBackend):
    """Fake a Control Panel Backend, act as a dbus.Interface."""

    bus_name = gui.DBUS_BUS_NAME
    object_path = gui.DBUS_PREFERENCES_PATH
    iface = gui.DBUS_PREFERENCES_IFACE
    exposed_methods = ['account_info', 'devices_info']


class FakedSessionBus(object):
    """Fake a session bus."""

    def get_object(self, bus_name, object_path, introspect=True,
                   follow_name_owner_changes=False, **kwargs):
        """Return a faked proxy for the given remote object."""
        return None


class FakedInterface(object):
    """Fake a dbus interface."""

    def __new__(cls, obj, dbus_interface, *args, **kwargs):
        if dbus_interface == gui.DBUS_PREFERENCES_IFACE:
            return FakedControlPanelBackend(obj, dbus_interface,
                                            *args, **kwargs)
        if dbus_interface == gui.ubuntu_sso.DBUS_CREDENTIALS_IFACE:
            return FakedSSOBackend(obj, dbus_interface, *args, **kwargs)


class LoadingTestCase(TestCase):
    """Test suite for the Loading widget (a spinner plus a label)."""

    def setUp(self):
        self.loading = gui.Loading()

    def test_creation(self):
        """A Loading can be created."""
        self.assertEqual(self.loading.label.get_text(), gui.LOADING)
        self.assertEqual(self.loading.spinner.get_property('active'), True)

    def test_visibility(self):
        """All widgets are visible."""
        self.assertTrue(self.loading.label.get_visible())
        self.assertTrue(self.loading.spinner.get_visible())

    def test_children(self):
        """A Loading can be created."""
        children = self.loading.get_children()
        self.assertEqual(2, len(children))
        self.assertTrue(self.loading.spinner is children[0])
        self.assertTrue(self.loading.label is children[-1])


class LabelLoadingTestCase(TestCase):
    """Test suite for the LabelLoading widget.

    (a label that shows a Loading until the text is set).

    """

    def setUp(self):
        self.widget = gui.LabelLoading()

    def test_creation(self):
        """A LabelLoading can be created."""
        self.assertEqual(self.widget.label.get_text(), '')
        self.assertFalse(self.widget.label.get_visible())

        self.assertIsInstance(self.widget.loading, gui.Loading)
        self.assertTrue(self.widget.loading.get_visible())
        self.assertTrue(self.widget.active)

    def test_stop(self):
        """Stop hides the Loading widget."""
        self.widget.stop()
        self.assertTrue(self.widget.label.get_visible())
        self.assertFalse(self.widget.loading.get_visible())
        self.assertFalse(self.widget.active)

    def test_start(self):
        """Start shows the Loading widget."""
        self.widget.start()
        self.assertFalse(self.widget.label.get_visible())
        self.assertTrue(self.widget.loading.get_visible())
        self.assertTrue(self.widget.active)

    def test_children(self):
        """A LabelLoading have proper children."""
        children = self.widget.get_children()
        self.assertEqual(2, len(children))
        self.assertTrue(self.widget.label is children[0])
        self.assertTrue(self.widget.loading is children[-1])

    def test_get_text(self):
        """Text can be queried."""
        text = 'Test me.'
        self.widget.label.set_text(text)
        self.assertEqual(self.widget.get_text(), text)

    def test_set_text(self):
        """Text can be set."""
        text = 'Test me.'
        self.widget.set_text(text)
        self.assertEqual(self.widget.label.get_text(), text)

    def test_get_label(self):
        """Label (markup text) can be queried."""
        text = 'Test <b>me</b>.'
        self.widget.label.set_markup(text)
        self.assertEqual(self.widget.get_label(), text)

    def test_set_markup(self):
        """Markup can be set."""
        text = 'Test <b>me</b>.'
        self.widget.set_markup(text)
        self.assertEqual(self.widget.label.get_label(), text)


class BaseTestCase(TestCase):
    """Basics for testing."""

    # self.klass is not callable
    # pylint: disable=E1102
    klass = None
    kwargs = {}

    def setUp(self):
        self.patch(gui.gtk, 'main', lambda: None)
        self.patch(gui.dbus, 'SessionBus', FakedSessionBus)
        self.patch(gui.dbus, 'Interface', FakedInterface)
        self.patch(gui.networkstate, 'NetworkManagerState', FakedNMState)

        self._called = False
        if self.klass is not None:
            self.ui = self.klass(**self.kwargs)

        self.memento = testcase.MementoHandler()
        self.memento.setLevel(logging.DEBUG)
        gui.logger.addHandler(self.memento)

    def tearDown(self):
        self._called = False
        try:
            self.ui.hide()
            del self.ui
            self.ui = None
        except AttributeError:
            pass

    def _set_called(self, *args, **kwargs):
        """Store 'args' and 'kwargs' for test assertions."""
        self._called = (args, kwargs)

    def assert_image_equal(self, image, filename):
        """Check that expected and actual represent the same image."""
        pb = gui.gtk.gdk.pixbuf_new_from_file(gui.get_data_file(filename))
        self.assertEqual(image.get_pixbuf().get_pixels(), pb.get_pixels())

    def assert_messages_equal(self, widget, msgs):
        """Check that 'widget' has all the entries in 'msgs'."""
        children = list(reversed(widget.get_children()))
        self.assertEqual(len(children), len(msgs))
        for label, msg in zip(reversed(children), msgs):
            full_msg = self.ui.BULLET + ' ' + msg
            self.assertTrue(label.get_visible())
            self.assertEqual(label.get_property('xalign'), 0)
            self.assertEqual(label.get_property('wrap'), True)
            self.assertEqual(label.get_label(), full_msg)

            expected = gui.gtk.Label()
            expected.set_markup(full_msg)
            self.assertEqual(label.get_text(), expected.get_text())

    def assert_backend_called(self, method_name, args, backend=None):
        """Check that the control panel backend 'method_name' was called."""
        if backend is None:
            backend = self.ui.backend
        self.assertIn(method_name, backend._called)
        self.assertEqual(backend._called[method_name], (args, {}))

    def assert_warning_correct(self, warning, text):
        """Check that 'warning' is visible, showing 'text'."""
        self.assertTrue(warning.get_visible(), 'Must be visible.')
        self.assertEqual(warning.get_text(), text)
        self.assertEqual(warning.get_label(), gui.WARNING_MARKUP % text)

    def assert_function_decorated(self, decorator, func):
        """Check that 'func' is decorated with 'decorator'."""
        expected = decorator(lambda: None)
        self.assertEqual(expected.func_code, func.im_func.func_code)


class ControlPanelMixinTestCase(BaseTestCase):
    """The test suite for the control panel widget."""

    klass = gui.ControlPanelMixin
    ui_filename = None

    def test_is_a_control_panel_mixin(self):
        """Inherits from ControlPanelMixin."""
        self.assertIsInstance(self.ui, gui.ControlPanelMixin)

    def test_ui_can_be_created(self):
        """UI main class exists and can be created."""
        self.assertTrue(self.ui is not None)

    def test_ui_file_is_read_at_startup(self):
        """The ui file is read and parse at startup."""
        if self.ui_filename is not None:
            self.patch(gui.gtk, 'Builder', FakedBuilder)
            ui = self.klass()
            self.assertEqual(ui.builder.filename,
                             gui.get_data_file(self.ui_filename))

    def test_gtk_signals_are_connected_at_startup(self):
        """The signals are connectedt at startup."""
        if self.ui_filename is not None:
            self.patch(gui.gtk, 'Builder', FakedBuilder)
            ui = self.klass()
            self.assertEqual(ui.builder.signals, ui)


class ControlPanelWindowTestCase(BaseTestCase):
    """The test suite for the control panel window."""

    klass = gui.ControlPanelWindow

    def test_is_a_window(self):
        """Inherits from gtk.Window."""
        self.assertIsInstance(self.ui, gui.gtk.Window)

    def test_startup_visibility(self):
        """The widget is visible at startup."""
        self.assertTrue(self.ui.get_visible(), 'must be visible at startup.')

    def test_main_start_gtk_main_loop(self):
        """The GTK main loop is started when calling main()."""
        self.patch(gui.gtk, 'main', self._set_called)
        self.ui.main()
        self.assertEqual(self._called, ((), {}), 'gtk.main was called.')

    def test_closing_stops_the_main_lopp(self):
        """The GTK main loop is stopped when closing the window."""
        self.patch(gui.gtk, 'main_quit', self._set_called)
        self.ui.emit('delete-event', None)
        self.assertEqual(self._called, ((), {}), 'gtk.main_quit was called.')

    def test_title_is_correct(self):
        """The window title is correct."""
        expected = self.ui.TITLE % {'app_name': gui.U1_APP_NAME}
        self.assertEqual(self.ui.get_title(), expected)

    def test_control_panel_is_the_only_child(self):
        """The control panel is the window's content."""
        children = self.ui.get_children()
        self.assertEqual(1, len(children))

        control_panel = self.ui.get_children()[0]
        self.assertTrue(control_panel is self.ui.control_panel)
        self.assertIsInstance(self.ui.control_panel, gui.ControlPanel)
        self.assertTrue(self.ui.control_panel.get_visible())

    def test_window_id_is_passed_to_child(self):
        """The child gets the window_id."""
        assert self.ui.window is not None
        self.assertEqual(self.ui.control_panel._window_id,
                         self.ui.window.xid)

    def test_icon_name_is_correct(self):
        """The icon name is correct."""
        self.assertEqual(self.ui.get_icon_name(), 'ubuntuone')

    def test_max_size(self):
        """Max size is not bigger than 966x576 (LP: #645526)."""
        self.assertTrue(self.ui.get_size_request() <= (966, 576))


class ControlPanelTestCase(ControlPanelMixinTestCase):
    """The test suite for the control panel itself."""

    klass = gui.ControlPanel
    kwargs = {'window_id': 7}
    ui_filename = 'controlpanel.ui'

    def test_is_a_vbox(self):
        """Inherits from gtk.VBox."""
        self.assertIsInstance(self.ui, gui.gtk.VBox)

    def test_startup_visibility(self):
        """The widget is visible at startup."""
        self.assertTrue(self.ui.itself.get_visible(),
                        'must be visible at startup.')

    def test_overview_is_shown_at_startup(self):
        """The overview is shown at startup."""
        self.assertIsInstance(self.ui.overview, gui.OverviewPanel)

    def test_window_id_is_passed_to_child(self):
        """The child gets the window_id."""
        self.assertEqual(self.ui.overview._window_id, self.kwargs['window_id'])

    def test_on_show_management_panel(self):
        """A ManagementPanel is shown when the callback is executed."""
        self.ui.on_show_management_panel()
        children = self.ui.get_children()
        self.assertIsInstance(children[-1], gui.ManagementPanel)


class UbuntuOneBinTestCase(ControlPanelMixinTestCase):
    """The test suite for a Ubuntu One page."""

    klass = gui.UbuntuOneBin
    ui_filename = 'page.ui'

    def test_is_a_greyable_bin(self):
        """Inherits from GreyableBin."""
        self.assertIsInstance(self.ui, gui.GreyableBin)

    def test_inner_widget_is_packed(self):
        """The 'itself' vbox is packed into the widget."""
        self.assertIn(self.ui.itself, self.ui.get_children())

    def test_startup_visibility(self):
        """The widget is visible at startup."""
        self.assertTrue(self.ui.get_visible(),
                        'must be visible at startup if credentials not found.')

    def test_warning_label_is_hidden(self):
        """The warning label is not shown by default."""
        self.assertFalse(self.ui.warning_label.get_visible())

    def test_image_is_correct(self):
        """There is an image attribute and is correct."""
        self.assert_image_equal(self.ui.image, 'overview.png')


class OverwiewPanelTestCase(ControlPanelMixinTestCase):
    """The test suite for the overview panel."""

    klass = gui.OverviewPanel
    kwargs = {'messages': None, 'window_id': 8}
    ui_filename = 'overview.ui'

    def test_is_a_greyable_bin(self):
        """Inherits from GreyableBin."""
        self.assertIsInstance(self.ui, gui.GreyableBin)

    def test_inner_widget_is_packed(self):
        """The 'itself' vbox is packed into the widget."""
        self.assertIn(self.ui.itself, self.ui.get_children())

    def test_join_now_is_default(self):
        """The 'join_now' button is the default widget."""
        self.assertTrue(self.ui.join_now_button.get_property('can-default'))

    def test_sso_backend(self):
        """Has a correct SSO backend."""
        self.assertIsInstance(self.ui.sso_backend, FakedSSOBackend)

    def test_sso_backend_signals(self):
        """The proper signals are connected to the backend."""
        self.assertEqual(self.ui.sso_backend._signals['CredentialsFound'],
                         [self.ui.on_credentials_found])
        self.assertEqual(self.ui.sso_backend._signals['CredentialsNotFound'],
                         [self.ui.on_credentials_not_found])
        self.assertEqual(self.ui.sso_backend._signals['CredentialsError'],
                         [self.ui.on_credentials_error])
        self.assertEqual(self.ui.sso_backend._signals['AuthorizationDenied'],
                         [self.ui.on_authorization_denied])


class OverwiewNetworkStatePanelTestCase(OverwiewPanelTestCase):
    """The test suite for the overview panel regarding network state."""

    def test_network_state_is_created(self):
        """The network state is created."""
        self.assertIsInstance(self.ui.network_manager_state,
                              gui.networkstate.NetworkManagerState)
        self.assertEqual(self.ui.network_manager_state._kwargs['result_cb'],
                         self.ui.on_network_state_changed)

    def test_network_state_is_queried_at_startup(self):
        """The network state is asked to the NetworkManagerState."""
        self.assertTrue('find_online_state' in
                        self.ui.network_manager_state._called)

    def test_state_online(self):
        """Network connection is online."""
        self.ui.on_network_state_changed(gui.networkstate.ONLINE)
        # all green, no warning
        self.assertFalse(self.ui.warning_label.get_visible())
        self.assertTrue(self.ui.get_sensitive())

    def test_state_offline(self):
        """Network connection is offline."""
        self.ui.on_network_state_changed(gui.networkstate.OFFLINE)
        msg = self.ui.NETWORK_OFFLINE % {'app_name': gui.U1_APP_NAME}

        self.assert_warning_correct(self.ui.warning_label, msg)
        self.assertFalse(self.ui.get_sensitive())

    def test_state_unknown(self):
        """Network connection is unknown."""
        self.ui.on_network_state_changed(gui.networkstate.UNKNOWN)

        self.assert_warning_correct(self.ui.warning_label,
                                    self.ui.NETWORK_UNKNOWN)
        self.assertFalse(self.ui.get_sensitive())


class OverwiewPanelOnlineTestCase(OverwiewPanelTestCase):
    """The test suite for the overview panel."""

    def setUp(self):
        super(OverwiewPanelOnlineTestCase, self).setUp()
        self.ui.on_network_state_changed(gui.networkstate.ONLINE)

    def test_find_credentials_is_called(self):
        """Credentials are asked to SSO backend."""
        self.assert_backend_called('find_credentials', (gui.U1_APP_NAME, {}),
                                   backend=self.ui.sso_backend)

    def test_on_credentials_found(self):
        """Callback 'on_credentials_found' is correct."""
        self.ui.connect('credentials-found', self._set_called)

        self.ui.on_credentials_found(gui.U1_APP_NAME, TOKEN)

        self.assertFalse(self.ui.get_visible())
        self.assertEqual(self._called, ((self.ui, gui.U1_APP_NAME, TOKEN), {}))

    def test_on_credentials_not_found(self):
        """Callback 'on_credentials_not_found' is correct."""
        self.ui.on_credentials_not_found(gui.U1_APP_NAME)
        self.assertTrue(self.ui.get_visible())

    def test_on_credentials_error(self):
        """Callback 'on_credentials_error' is correct."""
        self.ui.on_credentials_error(gui.U1_APP_NAME, {})
        self.assertTrue(self.ui.get_visible())
        self.assert_warning_correct(self.ui.warning_label,
                                    self.ui.CREDENTIALS_ERROR)

    def test_on_authorization_denied(self):
        """Callback 'on_authorization_denied' is correct."""
        self.ui.on_authorization_denied(gui.U1_APP_NAME)
        self.assertTrue(self.ui.get_visible())
        self.assert_warning_correct(self.ui.warning_label,
                                    self.ui.AUTHORIZATION_DENIED)


class OverwiewPanelAppNameMismatchTestCase(OverwiewPanelTestCase):
    """The test suite for the overview panel when the app_name won't match."""

    NOT_U1_APP = 'Not ' + gui.U1_APP_NAME

    def test_filter_by_app_name(self):
        """The filter_by_app_name decorator is correct."""
        f = gui.filter_by_app_name(self._set_called)
        f(self.ui, self.NOT_U1_APP)
        self.assertFalse(self._called)
        self.assertTrue(self.memento.check_info('ignoring', self.NOT_U1_APP))

        args = ('test', object())
        kwargs = {'really': 'AWESOME'}
        f(self.ui, gui.U1_APP_NAME, *args, **kwargs)
        self.assertEqual(self._called,
                         ((self.ui, gui.U1_APP_NAME,) + args, kwargs))

    def test_on_credentials_found(self):
        """Callback 'on_credentials_found' is not executed."""
        self.assert_function_decorated(gui.filter_by_app_name,
                                       self.ui.on_credentials_found)

    def test_on_credentials_not_found(self):
        """Callback 'on_credentials_not_found' is not executed."""
        self.assert_function_decorated(gui.filter_by_app_name,
                                       self.ui.on_credentials_not_found)

    def test_on_credentials_error(self):
        """Callback 'on_credentials_error' is not executed."""
        self.assert_function_decorated(gui.filter_by_app_name,
                                       self.ui.on_credentials_error)

    def test_on_authorization_denied(self):
        """Callback 'on_authorization_denied' is not executed."""
        self.assert_function_decorated(gui.filter_by_app_name,
                                       self.ui.on_authorization_denied)


class OverwiewPanelNoCredsTestCase(OverwiewPanelTestCase):
    """The test suite for the overview panel when no credentials are found."""

    messages = None

    def setUp(self):
        self.kwargs['messages'] = self.messages
        super(OverwiewPanelNoCredsTestCase, self).setUp()
        self.ui.on_credentials_not_found(gui.U1_APP_NAME)

    def test_startup_visibility(self):
        """The widget is visible at startup."""
        self.assertTrue(self.ui.get_visible(),
                        'must be visible at startup if credentials not found.')

    def test_warning_label_is_hidden(self):
        """The warning label is not shown by default."""
        self.assertFalse(self.ui.warning_label.get_visible())

    def test_image_is_correct(self):
        """There is an image attribute and is correct."""
        self.assert_image_equal(self.ui.image, 'overview.png')

    def test_messages(self):
        """If no messages are passed, the default list is used."""
        self.assert_messages_equal(self.ui.messages, gui.OVERVIEW_MSGS)

    def test_join_now_is_default_widget(self):
        """The join now button is the default widget."""
        self.assertTrue(self.ui.join_now_button.get_property('can_default'))

    def test_join_now_button_clicked(self):
        """Test the 'join now' button callback."""
        self.ui.join_now_button.clicked()

        args = (gui.U1_APP_NAME,
                {gui.TC_URL_KEY: gui.U1_TC_URL,
                 gui.HELP_TEXT_KEY: gui.U1_DESCRIPTION,
                 gui.WINDOW_ID_KEY: str(self.kwargs['window_id']),
                 gui.PING_URL_KEY: gui.U1_PING_URL})
        self.assert_backend_called('register', args,
                                   backend=self.ui.sso_backend)

    def test_connect_button_clicked(self):
        """Test the 'join now' button callback."""
        self.ui.connect_button.clicked()

        args = (gui.U1_APP_NAME,
                {gui.TC_URL_KEY: gui.U1_TC_URL,
                 gui.HELP_TEXT_KEY: gui.U1_DESCRIPTION,
                 gui.WINDOW_ID_KEY: str(self.kwargs['window_id']),
                 gui.PING_URL_KEY: gui.U1_PING_URL})
        self.assert_backend_called('login', args,
                                   backend=self.ui.sso_backend)

    def test_join_now_button_clicked_set_greyed(self):
        """Clicking on 'join_now' self is greyed."""
        self.ui.join_now_button.clicked()
        self.assertTrue(self.ui.get_property('greyed'), 'Must be greyed.')

    def test_join_now_button_clicked_removes_warning(self):
        """Clicking on 'join_now' the warnings are removed."""
        self.ui.on_authorization_denied(gui.U1_APP_NAME)  # show warning
        self.ui.join_now_button.clicked()

        self.assertFalse(self.ui.warning_label.get_visible())

    def test_connect_button_clicked_set_greyed(self):
        """Clicking on 'connect' self is greyed."""
        self.ui.connect_button.clicked()
        self.assertTrue(self.ui.get_property('greyed'), 'Must be greyed.')

    def test_connect_button_clicked_removes_warning(self):
        """Clicking on 'connect' the warnings are removed."""
        self.ui.on_authorization_denied(gui.U1_APP_NAME)  # show warning
        self.ui.connect_button.clicked()

        self.assertFalse(self.ui.warning_label.get_visible())

    def test_on_credentials_not_found_unset_greyed(self):
        """Callback 'on_credentials_not_found' unsets the 'greyed' prop."""
        self.ui.connect_button.clicked()
        self.ui.on_credentials_not_found(gui.U1_APP_NAME)

        self.assertFalse(self.ui.get_property('greyed'), 'Must not be greyed.')

    def test_on_credentials_error_unset_greyed(self):
        """Callback 'on_credentials_error' unsets the 'greyed' prop."""
        self.ui.connect_button.clicked()
        self.ui.on_credentials_error(gui.U1_APP_NAME, {})

        self.assertFalse(self.ui.get_property('greyed'), 'Must not be greyed.')

    def test_on_authorization_denied_unset_greyed(self):
        """Callback 'on_authorization_denied' unsets the 'greyed' prop."""
        self.ui.connect_button.clicked()
        self.ui.on_authorization_denied(gui.U1_APP_NAME)

        self.assertFalse(self.ui.get_property('greyed'), 'Must not be greyed.')

    def test_buttons_disabled_when_greyed(self):
        """Buttons should be disabled when widget is greyed."""
        self.ui.set_sensitive(True)
        self.ui.set_property('greyed', True)

        self.assertFalse(self.ui.join_now_button.is_sensitive())
        self.assertFalse(self.ui.connect_button.is_sensitive())

    def test_buttons_enabled_when_not_greyed(self):
        """Buttons should be enabled when widget is not greyed."""
        self.ui.set_sensitive(False)
        self.ui.set_property('greyed', False)

        self.assertTrue(self.ui.join_now_button.is_sensitive())
        self.assertTrue(self.ui.connect_button.is_sensitive())


class OverwiewPanelMessagesTestCase(OverwiewPanelTestCase):
    """The test suite for the overview panel when messages are set."""

    messages = ['Test me', 'A little bit more']


class OverwiewPanelMarkupMessagesTestCase(OverwiewPanelTestCase):
    """The test suite for the overview panel when markup messages are set."""

    messages = ['<small>Test me</small>', 'A <b>little</b> bit more']


class AccountTestCase(ControlPanelMixinTestCase):
    """The test suite for the account panel."""

    klass = gui.AccountPanel
    ui_filename = 'account.ui'

    def assert_account_info_correct(self, info):
        """Check that the displayed account info matches 'info'."""
        self.assertEqual(self.ui.name_label.get_text(),
                         FAKE_ACCOUNT_INFO['name'])
        self.assertEqual(self.ui.type_label.get_text(),
                         FAKE_ACCOUNT_INFO['type'])
        self.assertEqual(self.ui.email_label.get_text(),
                         FAKE_ACCOUNT_INFO['email'])

    def test_is_a_vbox(self):
        """Inherits from gtk.VBox."""
        self.assertIsInstance(self.ui, gui.gtk.VBox)

    def test_inner_widget_is_packed(self):
        """The 'itself' vbox is packed into the widget."""
        self.assertIn(self.ui.itself, self.ui.get_children())

    def test_is_visible(self):
        """Is visible."""
        self.assertTrue(self.ui.get_visible())

    def test_type_label_is_loading(self):
        """Placeholder for type label is a Loading widget."""
        self.assertIsInstance(self.ui.type_label, gui.LabelLoading)
        self.assertIn(self.ui.type_label, self.ui.type_box.get_children())

    def test_email_label_is_loading(self):
        """Placeholder for email label is a Loading widget."""
        self.assertIsInstance(self.ui.email_label, gui.LabelLoading)
        self.assertIn(self.ui.email_label, self.ui.email_box.get_children())

    def test_backend_signals(self):
        """The proper signals are connected to the backend."""
        self.assertEqual(self.ui.backend._signals['AccountInfoReady'],
                         [self.ui.on_account_info_ready])
        self.assertEqual(self.ui.backend._signals['AccountInfoError'],
                         [self.ui.on_account_info_error])

    def test_on_account_info_ready(self):
        """Account info is processed when ready."""
        self.ui.on_account_info_ready(FAKE_ACCOUNT_INFO)
        self.assert_account_info_correct(FAKE_ACCOUNT_INFO)

        for widget in (self.ui.name_label, self.ui.type_label,
                       self.ui.email_label):
            self.assertFalse(widget.active)

    def test_on_account_info_error(self):
        """Account info couldn't be retrieved."""
        self.ui.on_account_info_error()
        for widget in (self.ui.name_label, self.ui.type_label,
                       self.ui.email_label):
            self.assert_warning_correct(widget, self.ui.VALUE_ERROR)
            self.assertFalse(widget.active)


class ContactsTestCase(ControlPanelMixinTestCase):
    """The test suite for the contacts panel."""

    klass = gui.ContactsPanel
    ui_filename = 'contacts.ui'

    def test_is_a_vbox(self):
        """Inherits from gtk.VBox."""
        self.assertIsInstance(self.ui, gui.gtk.VBox)

    def test_inner_widget_is_packed(self):
        """The 'itself' vbox is packed into the widget."""
        self.assertIn(self.ui.itself, self.ui.get_children())

    def test_is_visible(self):
        """Is visible."""
        self.assertTrue(self.ui.get_visible())


class DevicesTestCase(ControlPanelMixinTestCase):
    """The test suite for the devices panel."""

    klass = gui.DevicesPanel
    ui_filename = 'devices.ui'

    def test_is_a_vbox(self):
        """Inherits from gtk.VBox."""
        self.assertIsInstance(self.ui, gui.gtk.VBox)

    def test_inner_widget_is_packed(self):
        """The 'itself' vbox is packed into the widget."""
        self.assertIn(self.ui.itself, self.ui.get_children())

    def test_is_visible(self):
        """Is visible."""
        self.assertTrue(self.ui.get_visible())


class ApplicationsTestCase(ControlPanelMixinTestCase):
    """The test suite for the applications panel."""

    klass = gui.ApplicationsPanel
    ui_filename = 'applications.ui'

    def test_is_a_vbox(self):
        """Inherits from gtk.VBox."""
        self.assertIsInstance(self.ui, gui.gtk.VBox)

    def test_inner_widget_is_packed(self):
        """The 'itself' vbox is packed into the widget."""
        self.assertIn(self.ui.itself, self.ui.get_children())

    def test_is_visible(self):
        """Is visible."""
        self.assertTrue(self.ui.get_visible())


class ManagementPanelTestCase(ControlPanelMixinTestCase):
    """The test suite for the management panel."""

    klass = gui.ManagementPanel
    ui_filename = 'management.ui'

    def assert_account_info_correct(self, info):
        """Check that the displayed account info matches 'info'."""
        used = int(info['quota_used'])
        total = int(info['quota_total'])
        expected = {'used': self.ui.humanize(used),
                    'total': self.ui.humanize(total),
                    'percentage': (used / total) * 100}
        self.assertEqual(self.ui.quota_label.get_text(),
                         self.ui.QUOTA_LABEL % expected)

    def test_is_a_vbox(self):
        """Inherits from gtk.VBox."""
        self.assertIsInstance(self.ui, gui.gtk.VBox)

    def test_startup_visibility(self):
        """The widget is visible at startup."""
        self.assertTrue(self.ui.get_visible(),
                        'must be visible at startup.')

    def test_inner_widget_is_packed(self):
        """The 'itself' vbox is packed into the widget."""
        self.assertIn(self.ui.itself, self.ui.get_children())

    def test_tabs_are_not_shown(self):
        """Tabs are not shown."""
        self.assertFalse(self.ui.notebook.get_show_tabs())

    def test_default_page_is_account(self):
        """The default page is Account."""
        self.assertEqual(self.ui.notebook.get_current_page(),
                         self.ui.ACCOUNT_PAGE)
        self.assertTrue(self.ui.account_button.get_active())

    def test_buttons_set_notebook_pages(self):
        """The notebook pages are set when clicking buttons."""
        msg = 'Page num should be %i when %s was clicked (got %i instead).'
        for tab in self.ui.tabs:
            button = '%s_button' % tab
            getattr(self.ui, button).clicked()
            expected = getattr(self.ui, ('%s_page' % tab).upper())
            actual = self.ui.notebook.get_current_page()
            self.assertEqual(actual, expected,
                             msg % (expected, button, actual))

    def test_buttons_indicates_current_page(self):
        """Only one button is activated at a time."""
        msg = 'Only button %s should be active (%s was active as well).'
        for tab in self.ui.tabs:
            button = '%s_button' % tab
            getattr(self.ui, button).clicked()
            for other in self.ui.tabs:
                if other is tab:
                    continue
                active = getattr(self.ui, '%s_button' % other).get_active()
                self.assertFalse(active, msg % (button, other))


class ManagementPanelAccountTestCase(ManagementPanelTestCase):
    """The test suite for the management panel (account tab)."""

    def test_backend_signals(self):
        """The proper signals are connected to the backend."""
        self.assertEqual(self.ui.backend._signals['AccountInfoReady'],
                         [self.ui.on_account_info_ready])
        self.assertEqual(self.ui.backend._signals['AccountInfoError'],
                         [self.ui.on_account_info_error])

    def test_account_info_is_requested(self):
        """Account info is requested to the backend."""
        self.assert_backend_called('account_info', ())

    def test_account_panel_is_packed(self):
        """The account panel is packed."""
        self.assertIsInstance(self.ui.account, gui.AccountPanel)
        actual = self.ui.notebook.get_nth_page(self.ui.ACCOUNT_PAGE)
        self.assertTrue(self.ui.account is actual)

    def test_contacts_panel_is_packed(self):
        """The contacts panel is packed."""
        self.assertIsInstance(self.ui.contacts, gui.ContactsPanel)
        actual = self.ui.notebook.get_nth_page(self.ui.CONTACTS_PAGE)
        self.assertTrue(self.ui.contacts is actual)

    def test_devices_panel_is_packed(self):
        """The devices panel is packed."""
        self.assertIsInstance(self.ui.devices, gui.DevicesPanel)
        actual = self.ui.notebook.get_nth_page(self.ui.DEVICES_PAGE)
        self.assertTrue(self.ui.devices is actual)

    def test_applications_panel_is_packed(self):
        """The applications panel is packed."""
        self.assertIsInstance(self.ui.applications, gui.ApplicationsPanel)
        actual = self.ui.notebook.get_nth_page(self.ui.APPLICATIONS_PAGE)
        self.assertTrue(self.ui.applications is actual)

    def test_placeholders_are_loading(self):
        """Placeholders for labels are a Loading widget."""
        widgets = (self.ui.quota_label, self.ui.status_label)
        for widget in widgets:
            self.assertIsInstance(widget, gui.LabelLoading)
            self.assertIn(widget, self.ui.status_box.get_children())

    def test_on_account_info_ready(self):
        """Account info is processed when ready."""
        self.ui.on_account_info_ready(FAKE_ACCOUNT_INFO)
        self.assert_account_info_correct(FAKE_ACCOUNT_INFO)

        for widget in (self.ui.quota_label,):
            self.assertFalse(widget.active)

    def test_on_account_info_error(self):
        """Account info couldn't be retrieved."""
        self.ui.on_account_info_error()
        for widget in (self.ui.quota_label,):
            self.assert_warning_correct(widget, self.ui.VALUE_ERROR)
            self.assertFalse(widget.active)
